#include "oXs_bmp280.h"

#ifdef DEBUG
//#define DEBUGI2CMS5611
//#define DEBUGI2CBMP280                // nach I2C Addr. suchen
//#define DEBUGBMP280                   // sensor data
//#define DEBUGDATA
//#define DEBUGVARIOI2C
#endif

extern unsigned long micros( void ) ;
extern unsigned long millis( void ) ;
extern void delay(unsigned long ms) ;

static bmp280_calib_data _bmp280_coeffs;   // Last read accelerometer data will be available here


//long result ;


#ifdef DEBUG  
OXS_BMP280::OXS_BMP280( HardwareSerial &print)
#else
OXS_BMP280::OXS_BMP280(void)
#endif
{
  // constructor
//  _addr=addr;
  varioData.SensorState = 0 ;
#ifdef DEBUG  
  printer = &print; //operate on the address of print
//  printer->begin(115200);
//  printer->print("Vario Sensor:MS5611 I2C Addr=");
//  printer->println(addr,HEX);
#endif
}


// **************** Setup the MS5611 sensor *********************
void OXS_BMP280::setup() {
  varioData.absoluteAlt.available = false ;
  varioData.relativeAlt.available = false ; 
  varioData.climbRate.available = false ;
  varioData.sensitivity.available = false ;
//  varioData.vSpeed10SecAvailable = false ;
  sensitivityMin = SENSITIVITY_MIN ; // set the min smoothing to the default value
  varioData.delaySmooth = 20000 ; // delay between 2 altitude calculation = 20msec = 20000 usec
  nextAltMillis  =  5000 ;  // in msec; save when Altitude has to be calculated; altitude is available only after some delay in order to get a stable value (less temperature drift)
//  nextAverageAltMillis =  nextAltMillis ;  // in msec ; save when AverageAltitude has to be calculated
//  nextAverageAltMillis =  nextAltMillis ; 

#ifdef ALT_TEMP_COMPENSATION
  alt_temp_compensation = ALT_TEMP_COMPENSATION ;
#endif

  
#ifdef DEBUG
  printer->print(F("Vario Sensor:BMP280 "));
  printer->println(" ");
  printer->print(F(" milli="));  
  printer->println(millis());

#endif
  
  I2c.begin() ;
  I2c.timeOut( 80); //initialise the time out in order to avoid infinite loop
#ifdef DEBUGI2CBMP280
  I2c.scan() ;
  printer->print(F("last I2C scan adr: "));
  printer->println( I2c.scanAdr , HEX  );
#endif  
    errorCalibration = false ;
    for (byte i = 1; i <=12; i++) {
       errorI2C =  I2c.read( BMP280_ADR, 0x86 + i*2, 2 ) ; //read 2 bytes from the device after sending the register to be read (first register = 0x88 (=register dig T1)
       if ( errorI2C > 0 ) {
#ifdef DEBUG
            printer->print(F("error code in setup I2CRead: "));
            printer->println( errorI2C );
#endif
            errorCalibration = true ;
        } else {
            low = I2c.receive() ;
            high = I2c.receive() ;
            _calibrationData[i] = high<<8 | low;
        }  
#ifdef DEBUG
        printer->print(F("calibration data #"));
        printer->print(i);
        printer->print(F(" = "));
        printer->print( _calibrationData[i] );
        printer->print(F(" error= "));
        printer->println( errorI2C );
#endif
    } // End for 

    _bmp280_coeffs.dig_T1  = _calibrationData[1];
    _bmp280_coeffs.dig_T2  = _calibrationData[2];
    _bmp280_coeffs.dig_T3  = _calibrationData[3];
    _bmp280_coeffs.dig_P1  = _calibrationData[4];
    _bmp280_coeffs.dig_P2  = _calibrationData[5];
    _bmp280_coeffs.dig_P3  = _calibrationData[6];
    _bmp280_coeffs.dig_P4  = _calibrationData[7];
    _bmp280_coeffs.dig_P5  = _calibrationData[8];
    _bmp280_coeffs.dig_P6  = _calibrationData[9];
    _bmp280_coeffs.dig_P7  = _calibrationData[10];
    _bmp280_coeffs.dig_P8  = _calibrationData[11];
    _bmp280_coeffs.dig_P9  = _calibrationData[12];

    I2c.write( BMP280_ADR , 0xF5 , (uint8_t)0 ) ; // t_sb = 0 ,IIR filte = 0
    I2c.write( BMP280_ADR , 0xF4 , 0x2F ) ; // osrs_t = 001 , osrs_p = 011 (4*), mode = 11
    
#ifdef DEBUG  
  printer->println(F("setup vario done."));  
#endif
  
}  //end of setup


//********************************************************************************************
//***                            read the sensor                                           ***
//********************************************************************************************
bool OXS_BMP280::readSensor() {
int32_t result = 0 ;
#ifdef  DEBUGVARIOI2C
    printer->print(F("sensorState= "));
    printer->println(varioData.SensorState);
#endif
  bool newVSpeedCalculated  = false ; 
  if (varioData.SensorState==0) { // ========================= Read the pressure
    if (  micros()   >   varioData.lastCommandMicros + 9000){ // wait 9 msec at least before asking for reading the pressure
//        long result = 0;
	      if(  ! I2c.read( BMP280_ADR, 0xF7, 3 )) { ; //read 3 bytes from the device starting from register F7 (presure); keep previous value in case of error 
        	result = I2c.receive() ;
         	result <<= 8 ;
         	result |= I2c.receive() ;
         	result <<= 8 ;
         	result |= I2c.receive() ;
                D1=result >> 4; // divide by 16 (20bit format)
        } else {
              D1 = 0 ; // D1 value are not processed to calculate Alt.
        }  
        
              if ( ! I2c.read( BMP280_ADR , 0xFA, 3 )) { ; //read 3 bytes from the device in register FA (temperature); keep previous value in case of error
                result = I2c.receive() ;
                result <<= 8 ;
         	result |= I2c.receive() ;
         	result <<= 8 ;
                result |= I2c.receive() ;
                D2=result >> 4; // divide by 16 (20bit format)
          }
        varioData.lastCommandMicros = micros(); 
        varioData.SensorState = 1;
    } // end of delay of 9 ms  
  } // end of SensorState == 1 
  else if (varioData.SensorState==1){ // =========================  
    if ( micros() > varioData.lastCommandMicros + 9000) { // wait 9000 usec to get Temp with high precision
      
          varioData.lastCommandMicros = micros() ; 
          varioData.SensorState=2; // 
    }  // End of process if temperature can be read 
  } // End of process if SensorState was 1  
  else if (varioData.SensorState==2) {    // ========================== new Pressure and (new or old) Temp are known so Request Pressure immediately and calculate altitude
    varioData.SensorState=0;
    if ((D1 > 0) & (millis() > 1000) ) { // If D1 has been read in a previous loop and if sensor is started since more than 1 sec, then calculate pressure etc...
              calculateVario() ;
              newVSpeedCalculated = true ;
    }
  } //   end ( varioData.SensorState == 2 )     
  return newVSpeedCalculated ;                             // return true if a new Vspeed is available  
}  // end read sensor     


void OXS_BMP280::calculateVario() {       
      if (D2Prev == 0) D2Prev = D2 ;
      D2Apply = (D2 + D2Prev ) / 2 ;
      D2Prev = D2 ; 

//*********************************************************************
// Returns temperature in DegC, resolution is 0.1 DegC. Output value of “512” equals 51.2 DegC.
// t_fine carries fine temperature as global value

      int32_t var1, var2;
      int32_t t_fine;                        
      
      var1 = ((((D2>>3) - ((int32_t)_bmp280_coeffs.dig_T1<<1))) * ((int32_t)_bmp280_coeffs.dig_T2)) >> 11;
      var2 = (((((D2>>4) - ((int32_t)_bmp280_coeffs.dig_T1)) * ((D2>>4) - ((int32_t)_bmp280_coeffs.dig_T1))) >> 12) *
            ((int32_t)_bmp280_coeffs.dig_T3)) >> 14;
      t_fine = var1 + var2;

      varioData.temperature = ((t_fine * 5 + 128) >> 8) / 10;  // = Temperature in 1/10°C
      
//*********************************************************************
      // calcul of pressure.

// Returns pressure in Pa as unsigned 64 bit integer. Output value of “963860000” equals 96386 Pa = 963.86 hPa

      int64_t varp1, varp2;
      uint64_t p;
      varp1 = ((int64_t)t_fine) - 128000;
      varp2 = varp1 * varp1 * (int64_t)_bmp280_coeffs.dig_P6;
      varp2 = varp2 + ((varp1*(int64_t)_bmp280_coeffs.dig_P5)<<17);
      varp2 = varp2 + (((int64_t)_bmp280_coeffs.dig_P4)<<35);
      varp1 = ((varp1 * varp1 * (int64_t)_bmp280_coeffs.dig_P3)>>8) + ((varp1 * (int64_t)_bmp280_coeffs.dig_P2)<<12);
      varp1 =(((((int64_t)1)<<47) + varp1))*((int64_t)_bmp280_coeffs.dig_P1)>>33;
      if (varp1 == 0)
        {
          p = 0; // avoid exception caused by division by zero
        } else {
          p = 1048576-D1;
          p = (((p<<31)-varp2)*3125)/varp1;
          varp1 = (((int64_t)_bmp280_coeffs.dig_P9) * (p>>13) * (p>>13)) >> 25;
          varp2 = (((int64_t)_bmp280_coeffs.dig_P8) * p) >> 19;
          p = ((p + varp1 + varp2) >> 8) + (((int64_t)_bmp280_coeffs.dig_P7)<<4);
        }
//return p;
//}
      varioData.rawPressure = p * 39;    // correct vale is p/256*1000 = 39,0625;

#ifdef DEBUGBMP280      
    dbg_delay += 1;  
    if (dbg_delay >=50) {
      dbg_delay = 0;  
      printer->print(F("T= "));
      printer->println(varioData.temperature);
      printer->println(D2);
      printer->print(F("P= "));
      printer->println((uint32_t)varioData.rawPressure);
      printer->println(D1);
      };
#endif  


//      varioData.temperature= TEMP;
//      OFF  = (((int64_t)_calibrationData[2]) << 16) + ( (_calibrationData[4] * dT) >> 7);
//      OFF  = (((int64_t)_calibrationData[2]) << 16) + ( ( (_calibrationData[4] - alt_temp_compensation ) * dT) >> 7);
//      SENS = (((int64_t)_calibrationData[1]) << 15) + ((_calibrationData[3] * dT) >> 8);
//      varioData.rawPressure= (((((((int64_t) D1) * (int64_t) SENS) >> 21) - OFF) * 10000 ) >> 15) ; // 1013.25 mb gives 1013250000 is a factor to keep higher precision (=1/100 cm).
      
      // altitude = 44330 * (1.0 - pow(pressure /sealevelPressure,0.1903));
      // other alternative (faster) = 1013.25 = 0 m , 954.61 = 500m , etc...
      //      Pressure	Alt (m)	Ratio
      //      101325	0	0.08526603
      //      95461	500	0.089525515
      //      89876	1000	0.094732853
      //      84598	1500	0.098039216
      //      79498	2000	0.103906899
      //      74686	2500	0.109313511
      //      70112	3000	0.115101289
      //      65768	3500	0.121270919
      //      61645	4000	0.127811861
      //      57733	4500	0.134843581
      //      54025	5000	

      uint32_t  rawPressure32;
      rawPressure32 = (uint32_t)varioData.rawPressure;
      
      if ( rawPressure32 > 954610000) {
        varioData.rawAltitude = ( 1013250000 - rawPressure32 ) * 0.08526603 ; // = 500 / (101325 - 95461)  // returned value 1234567 means 123,4567 m (temp is fixed to 15 degree celcius)
      } else if ( rawPressure32 > 898760000) {
        varioData.rawAltitude = 5000000 + ( 954610000 - rawPressure32 ) * 0.089525515  ; 
      } else if ( rawPressure32 > 845980000) {
        varioData.rawAltitude = 10000000 + ( 898760000 - rawPressure32 ) * 0.094732853  ; 
      } else if ( rawPressure32 > 794980000) {
        varioData.rawAltitude = 15000000 + ( 845980000 - rawPressure32 ) *  0.098039216 ; 
      } else if ( rawPressure32 > 746860000) {
        varioData.rawAltitude = 20000000 + ( 794980000 - rawPressure32 ) *  0.103906899 ; 
      } else if ( rawPressure32 > 701120000) {
        varioData.rawAltitude = 25000000 + ( 746860000 - rawPressure32 ) *  0.109313511 ; 
      } else if ( rawPressure32 > 657680000) {
        varioData.rawAltitude = 30000000 + ( 701120000 - rawPressure32 ) *  0.115101289 ; 
      } else if ( rawPressure32 > 616450000) {
        varioData.rawAltitude = 35000000 + ( 657680000 - rawPressure32 ) *  0.121270919 ; 
      } else if ( rawPressure32 > 577330000) {
        varioData.rawAltitude = 40000000 + ( 616450000 - rawPressure32 ) *  0.127811861 ;
      } else {    varioData.rawAltitude = 45000000 + ( 577330000 - rawPressure32 ) *  0.134843581 ;
      }
#ifdef DEBUGBMP280      
    if (dbg_delay ==1) { 
      printer->print(F("RP= "));
      printer->println(rawPressure32);
      printer->print(F("ALT= "));
      printer->println((uint32_t)varioData.rawAltitude);
      printer->println();
      };
#endif  
     
// here the classical way to calculate Vspeed with high and low pass filter      
      if (altitude == 0) {
        altitudeLowPass = altitudeHighPass = altitude = varioData.rawAltitude ;
//        pressureMicrosPrev2 = pressureMicrosPrev1 - 20000 ; 
      }
      altitude += 0.04 * (varioData.rawAltitude - altitude) ;
//      varioData.altitudeAt20MsecAvailable = true ; // inform openxsens.ino that calculation of dTE can be performed

      altitudeLowPass += 0.085 * ( varioData.rawAltitude - altitudeLowPass) ;
      altitudeHighPass += 0.1 * ( varioData.rawAltitude - altitudeHighPass) ;
//      if (pressureMicrosPrev1 > pressureMicrosPrev2 ) varioData.delaySmooth += 0.1 * ( pressureMicrosPrev1 -  pressureMicrosPrev2  - varioData.delaySmooth ) ; //delay between 2 measures  only if there is no overflow of pressureMicos
      climbRate2AltFloat = ((altitudeHighPass - altitudeLowPass )  * 5666.685 ) / 20000; 

      abs_deltaClimbRate =  abs(climbRate2AltFloat - varioData.climbRateFloat) ;
      if ( varioData.sensitivityPpm  > 0) sensitivityMin =   varioData.sensitivityPpm ; 
      if ( (abs_deltaClimbRate <= SENSITIVITY_MIN_AT) || (sensitivityMin >= SENSITIVITY_MAX) ) {
         varioData.sensitivity.value = sensitivityMin ;  
      } else if (abs_deltaClimbRate >= SENSITIVITY_MAX_AT)  {
         varioData.sensitivity.value = SENSITIVITY_MAX ; 
      } else {
         varioData.sensitivity.value = sensitivityMin + ( SENSITIVITY_MAX - sensitivityMin ) * (abs_deltaClimbRate - SENSITIVITY_MIN_AT) / (SENSITIVITY_MAX_AT - SENSITIVITY_MIN_AT) ;
      }
      varioData.climbRateFloat += varioData.sensitivity.value * (climbRate2AltFloat - varioData.climbRateFloat)  * 0.001 ; // sensitivity is an integer and must be divided by 1000
      
      if ( abs((int32_t)  varioData.climbRateFloat - varioData.climbRate.value) > VARIOHYSTERESIS ) {
          varioData.climbRate.value = (int32_t)  varioData.climbRateFloat  ;
      }    
      varioData.climbRate.available=true; // allows SPORT protocol to transmit the value
//      varioData.switchClimbRateAvailable = true ; // inform readsensors() that a switchable vspeed is available
//      varioData.averageClimbRateAvailable = true ; // inform readsensors() that a vspeed is available to calculate the average
      // AltitudeAvailable is set to true only once every 100 msec in order to give priority to climb rate on SPORT
      altMillis = millis() ;
      if (altMillis > nextAltMillis){
        nextAltMillis = altMillis + 100 ;
        varioData.absoluteAlt.value = altitude / 100 ; // altitude is in m *10000 and AbsoluteAlt must be in m * 100
        varioData.absoluteAlt.available=true ;  // Altitude is considered as available only after several loop in order to reduce number of transmission on Sport.
        varioData.sensitivity.available = true ;
        if (varioData.altOffset == 0) varioData.altOffset = varioData.absoluteAlt.value ;
        varioData.relativeAlt.value = varioData.absoluteAlt.value - varioData.altOffset ;
        varioData.relativeAlt.available = true ;
        if ( varioData.relativeAlt.value > varioData.relativeAltMax ) varioData.relativeAltMax = varioData.relativeAlt.value ;
        varioData.relativeAltMaxAvailable = true ;
//        if ( altMillis > nextAverageAltMillis ){ // calculation of the difference of altitude (in m) between the 10 last sec
//            nextAverageAltMillis = altMillis + 500 ; // calculate only once every 500 msec
//            varioData.vSpeed10Sec = (varioData.absoluteAlt.value - varioData.prevAlt[varioData.idxPrevAlt]) /100 ;
//            varioData.prevAlt[varioData.idxPrevAlt] = varioData.absoluteAlt.value ;
//            varioData.idxPrevAlt++ ;
//            if ( varioData.idxPrevAlt >= 20 ) varioData.idxPrevAlt = 0 ;
//            if ( altMillis > 15000) {  // make the data avalaible only after 15 sec)
//                varioData.vSpeed10SecAvailable = true ;
//            }  
//        }  

      } // end If (altMillis > nextAltMillis)
#ifdef DEBUGDATA
      static bool firstPrintAlt = true ;
      if (firstPrintAlt == true) {
          firstPrintAlt = false ;
//          printer->println(F( "T,Ra,Sm,A,NC,DS,AHP,ALP,CR2, Temp" )) ;
          printer->println(F( "T,Ra,Alt,vpsd, Alt2, rawVspd, vspd2 , smoothAlt, smoothVspd" )) ;
      }    
            printer->print(  pressureMicrosPrev1 ) ; printer->print(",");
            printer->print(  (float) varioData.rawAltitude  ) ; printer->print(","); // alt is displayed in CM with 2 decimal
 //           printer->print(  expoSmooth ) ;             printer->print(" ,");
            printer->print( (float) altitude  ) ;             printer->print(" ,");
            printer->print( varioData.climbRate ) ;            printer->print(" ,"); 
 //           printer->print( delaySmooth ) ;            printer->print(" ,"); 
 //           printer->print( altitudeHighPass ) ;             printer->print(" ,"); 
 //           printer->print( altitudeLowPass ) ;            printer->print(" ,"); 
 //           printer->print( climbRate2AltFloat ) ;             printer->print(" ,"); 
 //           printer->print( varioData.temperature ) ;
 //           printer->print( smoothAltitude ) ;            printer->print(" ,"); 
 //           printer->print( rawRateVSpeed ) ;            printer->print(" ,"); 
 //           printer->print( smoothRateVSpeed ) ;            printer->print(" ,"); 
 //           printer->print( expoSmooth5611_alt_auto * 1000 ) ;            printer->print(" ,"); 
 //           printer->print( expoSmooth5611_vSpeed_auto * 1000 ) ;            printer->print(" ,"); 
            printer->println( ) ;
            
#endif        

      pressureMicrosPrev2 = pressureMicrosPrev1 ;
} // End of calculate Vario






